﻿Param(
    [Parameter(Mandatory=$false, Position=0)]
      [switch]$setup
)
 
Function IS-InSubnet() 
{ 
 
[CmdletBinding()] 
[OutputType([bool])] 
Param( 
                    [Parameter(Mandatory=$true, 
                     ValueFromPipelineByPropertyName=$true, 
                     Position=0)] 
                    [validatescript({([System.Net.IPAddress]$_).AddressFamily -match 'InterNetwork'})] 
                    [string]$ipaddress="", 
                    [Parameter(Mandatory=$true, 
                     ValueFromPipelineByPropertyName=$true, 
                     Position=1)] 
                    [validatescript({(([system.net.ipaddress]($_ -split '/'|select -first 1)).AddressFamily -match 'InterNetwork') -and (0..32 -contains ([int]($_ -split '/'|select -last 1) )) })] 
                    [string]$Cidr="" 
    ) 
Begin{ 
        [int]$BaseAddress=[System.BitConverter]::ToInt32((([System.Net.IPAddress]::Parse(($cidr -split '/'|select -first 1))).GetAddressBytes()),0) 
        [int]$Address=[System.BitConverter]::ToInt32(([System.Net.IPAddress]::Parse($ipaddress).GetAddressBytes()),0) 
        [int]$mask=[System.Net.IPAddress]::HostToNetworkOrder(-1 -shl (32 - [int]($cidr -split '/' |select -last 1))) 
} 
Process{ 
        if( ($BaseAddress -band $mask) -eq ($Address -band $mask)) 
        { 
 
            $status=$True 
        }else { 
 
        $status=$False 
        } 
} 
end { Write-output $status } 
} 
function UserPositive( $userin)
{
    if( ($userin.ToLower() -eq "y") -or ($userin.ToLower() -eq "yes") )
        {
        return $true
        }
    else
        {
        return $false
        }
}

function ConfirmUserIn( $userin )
{
    if( ([string]::IsNullOrEmpty( $userin) -eq $false) -and ([string]::IsNullOrWhitespace( $userin) -eq $false) )
        {
        return $true
        }
    else
        {
        return $false
        }
}

function RoutesToArray( $routesin )
{
    $output = $routesin.Split(",")
    return $output

}

# script starts here

$settingsfilename = ".\DetectVpn.json"

if( $setup )
    {
    $settings = New-Object -TypeName PSObject
    $settings | Add-Member -MemberType NoteProperty -Name DetectMSVPN -Value $null   
    $settings | Add-Member -MemberType NoteProperty -Name MSVPNServerAddress -Value $null
    $settings | Add-Member -MemberType NoteProperty -Name VPNRoutes -Value $null
    $Settings | Add-Member -MemberType NoteProperty -Name ExcludedRoutes -Value $null

    # ask a series of questions
    Write-Host "`r`nRedstor Skip Backup VPN Detected Script - Setup Mode`r`n"
    # detect Microsoft VPN?
    $userin = Read-Host -Prompt "Abort Backup if on Microsoft VPN?[y/N]"
    if( UserPositive( $UserIn) )
        {
        $settings.DetectMSVPN = $true

        # if so, which VPN connection name?
        $userin = Read-Host -Prompt "Enter ServerAddress of connection or * for all connections"
        if( ConfirmUserIn($userin) ) 
            {
            $Settings.MSVPNServerAddress = $userin
            }
        else
            {
            Write-Error "No value entered for VPN Connection name."
            exit
            }

        
        }
    else
        {
        # if not microsoft VPN, detect by network route
        $userin = Read-Host -Prompt "Enter routes that match VPN connection in CIDR format, comma separated (eg 192.168.1.0/24,10.10.0.0/16)"
        if( ConfirmUserIn($Userin))
            {
            $temproutes = (RoutesToArray( $userin ))
            foreach( $temproute in $temproutes )
                {
                if( $temproute.IndexOf( "/" ) -lt 0 )
                    {
                    Write-Error "Entered route $temproute is not in CIDR format."
                    exit
                    }
                }
            
            $settings.VPNRoutes = $temproutes 
            }
        else
            {
            Write-Error "No value entered for VPN routes."
            exit
            }

        # are there any exclusions?
        $userin = Read-Host -prompt "Do you want to exclude any networks (back up anyway if they and the VPN routes are present)?[y/N]"
        if( UserPositive( $UserIn) )
            {
            $userin = Read-Host -Prompt "Enter excluded routes in CIDR format, comma separated"
            if( ConfirmUserIn($userin))
                {
                $temproutes = (RoutesToArray( $userin ))
                foreach( $temproute in $temproutes )
                    {
                    if( $temproute.IndexOf( "/" ) -lt 0 )
                        {
                        Write-Error "Entered route $temproute is not in CIDR format."
                        exit
                        }
                    }

                $settings.ExcludedRoutes = $temproutes
                }
            else
                {
                Write-Error "Exclusion option selected but no routes entered."
                exit
                }

            }
        }
        
        $settings | ConvertTo-Json | Set-Content $settingsfilename -Force
        Write-Output "Settings written to $settingsfilename"
    }
else
    {
    
    
    if( (Test-Path -path $settingsfilename ) -eq $true )
        {

        $settings = Get-Content -Path $settingsfilename -Raw | ConvertFrom-Json 

        if( $settings -eq $null )
            {
            Write-Error "$settingsfilename is invalid, please re-run setup."
            exit
            }

        $vpnconnected = $false
        
        # work out mode of operation

        if( $settings.DetectMSVPN -eq $true )
            {
            $vpnconnections = Get-VpnConnection | Where-Object {$_.ConnectionStatus -eq "Connected" }

            if( $vpnconnections )
                {

                if( $settings.MSVPNServerAddress -eq "*" )
                    {
                    Write-Output "Connected MSVPN Matched on wildcard"
                    $vpnconnected = $true
                    }
                else
                    {
                    Write-Output ( "Checking for MSVPN matching " + $settings.MSVPNServerAddress )
                    $matchvpn = $vpnconnections | Where-Object {$_.ServerAddress -eq $settings.MSVPNServerAddress }
                    if( $matchvpn )
                        {
                        Write-Output ( "Connected MSVPN found matching " + $settings.MSVPNServerAddress )
                        $vpnconnected = $true
                        }
                    }
                }

            }
        else
            {
            
            $vpnroutes = $settings.vpnRoutes
            $routes = Get-NetRoute    
            foreach( $vpnroute in $vpnroutes )
                {
                foreach( $route in $routes )
                    {
                    $localaddr = $route.DestinationPrefix
                    $pos = $localaddr.IndexOf( "/" )
                    $localaddr = $localaddr.Substring(0, $pos )

                    if( IS-InSubnet -ipaddress $localaddr -Cidr $vpnroute )
                        {
                        Write-Output "$localaddr is in $vpnroute VPN route"
                        $vpnconnected = $true
                        break
                        }
                    }

                if( $vpnconnected -eq $true )
                    {
                    break
                    } 
                }

            
            # if the vpn is connected, check if the routes are in the exclusions - if they are set the vpnconnected value back to false
            if( ($vpnconnected -eq $true) -and ($null -ne $settings.excludedRoutes))
                {
                Write-Output "VPN found, exclusions being checked."
                $excludedroutes = $settings.excludedRoutes
                $routes = Get-NetRoute

                foreach( $excludedroute in $excludedroutes )
                    {
                    foreach( $route in $routes )
                        {
                        $localaddr = $route.DestinationPrefix
                        $pos = $localaddr.IndexOf( "/" )
                        $localaddr = $localaddr.Substring(0, $pos )

                        if( IS-InSubnet -ipaddress $localaddr -Cidr $excludedroute )
                            {
                            Write-Output "$localaddr is in $excludedroute excluded route, overriding abort"
                            $vpnconnected = $false
                            break
                            }
                        }

                    if( $vpnconnected -eq $false )
                        {
                        break
                        } 
                    }
                }

            }

        # do we need to abort the backup?
        if( $vpnconnected -eq $true )
            {
            Write-Output "VPN detected."
            exit 1
            }
        else
            {
            Write-Output "VPN not detected."
            }    
        
        }
    else
        {
        Write-Error "Settings file $settingsfilename not found, run .\DetectVpn.ps1 -setup to configure settings."
        }
    


    }


    